/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2006  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 **********************************************************************/

#ifndef CLASS_GAMEPLAY_ACTION_HEADER
#define CLASS_GAMEPLAY_ACTION_HEADER

#include "../misc/translator.h"

class GameplayAction {
  public:
  enum Type {
    MARRIAGE,
    GENSCHER,
    POVERTY_SHIFT,
    POVERTY_RETURNED,
    POVERTY_DENIED_BY_ALL,
    ANNOUNCEMENT,
    SWINES,
    HYPERSWINES,
    CARD_PLAYED,
    TRICK_FULL
  }; // enum Type

    class Marriage;
    class Genscher;
    class PovertyShift;
    class PovertyReturned;
    class PovertyDeniedByAll;
    class Announcement;
    class Swines;
    class Hyperswines;
    class CardPlayed;
    class TrickFull;

  public:
    // read the data from the line
    static GameplayAction* new_(string const& line, istream& istr);

  protected:
    // constructor
    GameplayAction(Type const type, unsigned const playerno);

    // copy constructor
    GameplayAction(GameplayAction const& action); // default
    // copy operator
    GameplayAction& operator=(GameplayAction const& action); // default

  public:
    // destructor
    virtual ~GameplayAction();


    // write the action into the output stream
    virtual void write(ostream& ostr) const = 0;

    // a string representing the data
    virtual Translator::Translation data_translation() const;

  public:
    Type type; // which action
    unsigned player; // which player has made the action

  private: // unused
    GameplayAction();
}; // class GameplayAction

// the name of the action
string name(GameplayAction::Type const type);
// write the typ of the action in the output stream
ostream& operator<<(ostream& ostr, GameplayAction::Type const type);
// write the actio nin the output stream
ostream& operator<<(ostream& ostr, GameplayAction const& action);

#endif // #ifndef CLASS_GAMEPLAY_ACTION_HEADER
